@@ -173,11 +173,8 @@ impl Simd<f32> for f32x4 {
     }
 
     type ComplexArray = [Complex<f32>; 2];
-
-    #[inline]
-    fn len() -> usize {
-        4
-    }
+	
+	const LEN: usize = 4;
 
     #[inline]
     fn load_wrap_unchecked(array: &[f32], idx: usize) -> f32x4 {
@@ -294,6 +291,24 @@ impl Simd<f32> for f32x4 {
     fn sum_complex(&self) -> Complex<f32> {
         Complex::<f32>::new(self.0 + self.2, self.1 + self.3)
     }
+    
+    #[inline]
+    fn max(self, other: Self) -> Self {
+        f32x4::new(
+            f32::max(self.0, other.0),
+            f32::max(self.1, other.1),
+            f32::max(self.2, other.2),
+            f32::max(self.3, other.3))
+    }
+    
+    #[inline]
+    fn min(self, other: Self) -> Self {
+        f32x4::new(
+            f32::min(self.0, other.0),
+            f32::min(self.1, other.1),
+            f32::min(self.2, other.2),
+            f32::min(self.3, other.3))
+    }
 }
 
 impl Simd<f64> for f64x2 {
@@ -307,11 +322,8 @@ impl Simd<f64> for f64x2 {
     }
 
     type ComplexArray = [Complex<f64>; 1];
-
-    #[inline]
-    fn len() -> usize {
-        2
-    }
+	
+	const LEN: usize = 2;
 
     #[inline]
     fn load_wrap_unchecked(array: &[f64], idx: usize) -> f64x2 {
@@ -422,4 +434,18 @@ impl Simd<f64> for f64x2 {
     fn sum_complex(&self) -> Complex<f64> {
         Complex::<f64>::new(self.0, self.1)
     }
+    
+    #[inline]
+    fn max(self, other: Self) -> Self {
+        f64x2::new(
+            f64::max(self.0, other.0),
+            f64::max(self.1, other.1))
+    }
+    
+    #[inline]
+    fn min(self, other: Self) -> Self {
+        f64x2::new(
+            f64::min(self.0, other.0),
+            f64::min(self.1, other.1))
+    }
 }
