@@ -13,9 +13,9 @@ pub trait Simd<T>: Sized
     
     /// The type of complex valued array which matches a SIMD register.
     type ComplexArray;
-    
-    /// Number of elements in a SIMD register.
-    fn len() -> usize;
+	
+	/// Number of elements in a SIMD register.
+	const LEN: usize;
     
     /// Loads a SIMD register from an array. If the end of the array is approached
     /// then the load code wraps around to the beginning and starts to load the first 
@@ -71,6 +71,15 @@ pub trait Simd<T>: Sized
     /// Calculates the sum of all register elements, assuming that they
     /// are complex valued.
     fn sum_complex(&self) -> Complex<T>;
+    
+    fn max(self, other: Self) -> Self;
+    
+    fn min(self, other: Self) -> Self;
+}
+
+/// Dirty workaround since the stdsimd doesn't implement conversion traits (yet?).
+pub trait SimdFrom<T> {
+    fn regfrom(T) -> Self;
 }
 
 /// SIMD methods which share their implementation independent if it's a `f32` or `f64` register.
@@ -156,9 +165,16 @@ pub trait SimdApproximations<T>: Simd<T>
 /// Private struct copied over from the `simd` crate to implement the `load_unchecked` 
 /// and `store_unchecked` methods for the `SimdGeneric` trait.
 #[repr(packed)]
-#[derive(Debug, Copy, Clone)]
 struct Unalign<T>(T);
 
+impl<T: Copy> Copy for Unalign<T> { }
+
+impl<T: Clone + Copy> Clone for Unalign<T> {
+    fn clone(&self) -> Self {
+        *self
+    }
+} 
+
 macro_rules! simd_generic_impl {
     ($data_type:ident, $reg:ident)
     =>
@@ -175,11 +191,11 @@ macro_rules! simd_generic_impl {
                 let data_length = array.len();
                 let addr = array.as_ptr();
                 let scalar_left = (addr as usize % mem::size_of::<Self>()) / mem::size_of::<f32>();
-                if scalar_left + $reg::len() > data_length {
+                if scalar_left + $reg::LEN > data_length {
                     // Result order: scalar_left, scalar_right, vectorization_length
                     (data_length, data_length, 0)
                 } else {
-                    let right = (data_length - scalar_left) % Self::len();
+                    let right = (data_length - scalar_left) % Self::LEN;
                     (scalar_left, data_length - right, data_length - right)
                 }
             }
@@ -223,7 +239,7 @@ macro_rules! simd_generic_impl {
             fn array_to_regs(array: &[$data_type]) -> &[Self] {
                 unsafe {
                     let len = array.len();
-                    let reg_len = Self::len();
+                    let reg_len = Self::LEN;
                     if len % reg_len != 0 {
                         panic!("Argument must be dividable by {}", reg_len);
                     }
@@ -236,7 +252,7 @@ macro_rules! simd_generic_impl {
             fn array_to_regs_mut(array: &mut [$data_type]) -> &mut [Self] {
                 unsafe {
                     let len = array.len();
-                    let reg_len = Self::len();
+                    let reg_len = Self::LEN;
                     if len % reg_len != 0 {
                         panic!("Argument must be dividable by {}", reg_len);
                     }
@@ -286,14 +302,13 @@ mod avx;
 #[cfg(any(feature = "doc", feature="use_avx"))]
 pub use self::avx::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
 
-#[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
+#[cfg(any(feature = "doc", all(feature= "use_sse", not(feature= "use_avx"))))]
 mod sse;
 
 #[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
 pub use self::sse::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
 
-// #[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
-#[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
+#[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
 mod approximations;
 
 #[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
@@ -302,7 +317,7 @@ mod fallback;
 #[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
 pub use self::fallback::{Reg32, Reg64};
 
-#[cfg(any(feature = "doc", not(feature="use_sse")))]
+#[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
 mod approx_fallback;
 
 simd_generic_impl!(f32, Reg32);
