@@ -1,8 +1,7 @@
-use simd::{f32x4, i32x4, u32x4};
-use simd::x86::sse3::Sse3F32x4;
+use stdsimd::simd::*;
+use stdsimd::vendor::*;
 use numbers::*;
-use super::Simd;
-use simd::x86::sse2::{f64x2, i64x2, u64x2};
+use super::{Simd, SimdFrom};
 
 pub type Reg32 = f32x4;
 
@@ -16,6 +15,11 @@ pub type UIntReg32 = u32x4;
 
 pub type UIntReg64 = u64x2;
 
+/// This value must be read in groups of 2 bits:
+/// 10 means that the third position (since it's the third bit pair)
+/// will be replaced with the value of the second position (10b = 2d)
+const SWAP_IQ_PS: i32 = 0b10110001;
+
 impl Simd<f32> for f32x4 {
     type Array = [f32; 4];
 
@@ -27,11 +31,8 @@ impl Simd<f32> for f32x4 {
     }
 
     type ComplexArray = [Complex<f32>; 2];
-
-    #[inline]
-    fn len() -> usize {
-        4
-    }
+	
+	const LEN: usize = 4;
 
     #[inline]
     fn load_wrap_unchecked(array: &[f32], idx: usize) -> f32x4 {
@@ -70,13 +71,9 @@ impl Simd<f32> for f32x4 {
         let scaling_real = f32x4::splat(value.re);
         let scaling_imag = f32x4::splat(value.im);
         let parallel = scaling_real * self;
-        // There should be a shufps operation which shuffles the vector self
-        let shuffled = f32x4::new(self.extract(1),
-                                  self.extract(0),
-                                  self.extract(3),
-                                  self.extract(2));
+        let shuffled = unsafe { _mm_permute_ps(self, SWAP_IQ_PS) };
         let cross = scaling_imag * shuffled;
-        parallel.addsub(cross)
+        unsafe { _mm_addsub_ps(parallel, cross) }
     }
 
     #[inline]
@@ -92,15 +89,9 @@ impl Simd<f32> for f32x4 {
                                       value.extract(3));
 
         let parallel = scaling_real * self;
-        // There should be a shufps operation which shuffles the vector self
-        let shuffled = f32x4::new(self.extract(1),
-                                  self.extract(0),
-                                  self.extract(3),
-                                  self.extract(2));
-
-
+        let shuffled = unsafe { _mm_permute_ps(self, SWAP_IQ_PS) };
         let cross = scaling_imag * shuffled;
-        parallel.addsub(cross)
+        unsafe { _mm_addsub_ps(parallel, cross) }
     }
 
     #[inline]
@@ -109,46 +100,36 @@ impl Simd<f32> for f32x4 {
                                       self.extract(0),
                                       self.extract(2),
                                       self.extract(2));
+
         let scaling_real = f32x4::new(self.extract(1),
                                       self.extract(1),
                                       self.extract(3),
                                       self.extract(3));
+                                                      
         let parallel = scaling_real * value;
-        // There should be a shufps operation which shuffles the vector self
-        let shuffled = f32x4::new(value.extract(1),
-                                  value.extract(0),
-                                  value.extract(3),
-                                  value.extract(2));
-
+        let shuffled = unsafe { _mm_permute_ps(value, SWAP_IQ_PS) };
         let cross = scaling_imag * shuffled;
-        let mul = parallel.addsub(cross);
+        let mul = unsafe { _mm_addsub_ps(parallel, cross) };
         let square = shuffled * shuffled;
-        let square_shuffled = f32x4::new(square.extract(1),
-                                         square.extract(0),
-                                         square.extract(3),
-                                         square.extract(2));
-
+        let square_shuffled = unsafe { _mm_permute_ps(square, SWAP_IQ_PS) };
         let sum = square + square_shuffled;
         let div = mul / sum;
-        f32x4::new(div.extract(1),
-                   div.extract(0),
-                   div.extract(3),
-                   div.extract(2))
+        unsafe { _mm_permute_ps(div, SWAP_IQ_PS) }
     }
 
     #[inline]
     fn complex_abs_squared(self) -> f32x4 {
         let squared = self * self;
 
-        squared.hadd(squared)
+        unsafe { _mm_hadd_ps(squared, squared) }
     }
 
     #[inline]
     fn complex_abs(self) -> f32x4 {
         let squared = self * self;
 
-        let squared_sum = squared.hadd(squared);
-        squared_sum.sqrt()
+        let squared_sum = unsafe { _mm_hadd_ps(squared, squared) };
+        unsafe { _mm_sqrt_ps(squared_sum) }
     }
 
     #[inline]
@@ -173,20 +154,21 @@ impl Simd<f32> for f32x4 {
 
     #[inline]
     fn sqrt(self) -> f32x4 {
-        self.sqrt()
+        unsafe { _mm_sqrt_ps(self) }
     }
 
     #[inline]
     fn store_half_unchecked(self, target: &mut [f32], index: usize) {
         unsafe {
             *target.get_unchecked_mut(index) = self.extract(0);
-            *target.get_unchecked_mut(index + 1) = self.extract(1);
+            *target.get_unchecked_mut(index + 1) = self.extract(1);                             
         }
     }
 
     #[inline]
     fn sum_real(&self) -> f32 {
         self.extract(0) + self.extract(1) + self.extract(2) + self.extract(3)
+                                                                             
     }
 
     #[inline]
@@ -194,6 +176,16 @@ impl Simd<f32> for f32x4 {
         Complex::<f32>::new(self.extract(0) + self.extract(2),
                             self.extract(1) + self.extract(3))
     }
+    
+    #[inline]
+    fn max(self, other: Self) -> Self {
+        unsafe { _mm_max_ps(self, other) }
+    }
+    
+    #[inline]
+    fn min(self, other: Self) -> Self {
+        unsafe { _mm_min_ps(self, other) }
+    }
 }
 
 impl Simd<f64> for f64x2 {
@@ -207,11 +199,8 @@ impl Simd<f64> for f64x2 {
     }
 
     type ComplexArray = [Complex<f64>; 1];
-
-    #[inline]
-    fn len() -> usize {
-        2
-    }
+	
+	const LEN: usize = 2;
 
     #[inline]
     fn load_wrap_unchecked(array: &[f64], idx: usize) -> f64x2 {
@@ -258,12 +247,12 @@ impl Simd<f64> for f64x2 {
         let complex = Complex::new(self.extract(0), self.extract(1));
         let value = Complex::new(value.extract(0), value.extract(1));
         let result = complex * value;
-        f64x2::new(result.re, result.im)
+        f64x2::new(result.re, result.im)            
     }
 
     #[inline]
     fn div_complex(self, value: f64x2) -> f64x2 {
-        let complex = Complex::new(self.extract(0), self.extract(1));
+        let complex = Complex::new(self.extract(0), self.extract(1));       
         let value = Complex::new(value.extract(0), value.extract(1));
         let result = complex / value;
         f64x2::new(result.re, result.im)
@@ -284,8 +273,8 @@ impl Simd<f64> for f64x2 {
         let result = (a * a + b * b).sqrt();
         f64x2::new(result, 0.0)
     }
+    
     #[inline]
-
     fn complex_abs_squared2(self) -> f64x2 {
         self.complex_abs_squared()
     }
@@ -297,7 +286,7 @@ impl Simd<f64> for f64x2 {
 
     #[inline]
     fn sqrt(self) -> f64x2 {
-        f64x2::new(self.extract(0).sqrt(), self.extract(1).sqrt())
+        unsafe { _mm_sqrt_pd(self) }
     }
 
     #[inline]
@@ -316,4 +305,55 @@ impl Simd<f64> for f64x2 {
     fn sum_complex(&self) -> Complex<f64> {
         Complex::<f64>::new(self.extract(0), self.extract(1))
     }
+    
+    #[inline]
+    fn max(self, other: Self) -> Self {
+        unsafe { _mm_max_pd(self, other) }
+    }
+    
+    #[inline]
+    fn min(self, other: Self) -> Self {
+        unsafe { _mm_min_pd(self, other) }
+    }
 }
+
+impl SimdFrom<f32x4> for i32x4 {
+    fn regfrom(value: f32x4) -> Self {
+        value.as_i32x4()
+    }
+}
+
+impl SimdFrom<i32x4> for f32x4 {
+    fn regfrom(value: i32x4) -> Self {
+        value.as_f32x4()
+    }
+}
+
+impl SimdFrom<f64x2> for i64x2 {
+    fn regfrom(value: f64x2) -> Self {
+        value.as_i64x2()
+    }
+}
+
+impl SimdFrom<i64x2> for f64x2 {
+    fn regfrom(value: i64x2) -> Self {
+        value.as_f64x2()
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn shuffle_test() {
+        let vec = f32x4::new(1.0, 2.0, 3.0, 4.0);
+        let result = unsafe { _mm_permute_ps(vec, SWAP_IQ_PS) };
+        let expected = 
+            f32x4::new(vec.extract(1),
+                vec.extract(0),
+                vec.extract(3),
+                vec.extract(2));
+        assert_eq!(result, expected);
+    }
+}
\ No newline at end of file
