@@ -66,11 +66,11 @@ pub trait ModuloOps<T>
     fn unwrap(&mut self, divisor: T);
 }
 
-/// Recommended to be only used with feature flags `use_sse` or `use_avx`.
+/// Recommended to be only used with the CPU feature flags `sse` or `avx`.
 ///
 /// This trait provides alternative implementations for some standard functions which
 /// are less accurate but perform faster. Those approximations are written for SSE2
-/// (feature flag `use_sse`) or AVX2 (feature flag `use_avx`) processors. Without any of those
+/// (target feature flag `sse`) or AVX2 (target feature flag `avx`) processors. Without any of those
 /// feature flags the standard library functions will be used instead.
 ///
 /// Information on the error of the approximation and their performance are rough numbers.
@@ -330,7 +330,7 @@ impl<S, T, N, D> ApproximatedOps<T> for DspVec<S, T, N, D>
         let base_ln = T::Reg::splat(base.ln());
         self.simd_real_operation(|x, b| x.ln_approx() / b,
                                  |x, b| x.ln() / b.extract(0),
-                                 (base_ln),
+                                 base_ln,
                                  APPROX_COMPLEXITY);
     }
 
@@ -341,7 +341,7 @@ impl<S, T, N, D> ApproximatedOps<T> for DspVec<S, T, N, D>
         let base_ln = T::Reg::splat(base.ln());
         self.simd_real_operation(|y, x| (x * y).exp_approx(),
                                  |y, x| (x.extract(0) * y).exp(),
-                                 (base_ln),
+                                 base_ln,
                                  APPROX_COMPLEXITY);
     }
 
@@ -351,7 +351,7 @@ impl<S, T, N, D> ApproximatedOps<T> for DspVec<S, T, N, D>
         let exponent = T::Reg::splat(exponent);
         self.simd_real_operation(|x, y| (x.ln_approx() * y).exp_approx(),
                                  |x, y| (x.ln() * y.extract(0)).exp(),
-                                 (exponent),
+                                 exponent,
                                  APPROX_COMPLEXITY);
 
     }
