@@ -1,7 +1,7 @@
-use RealNumber;
-use super::super::{Owner, ToComplexResult, TransRes, Buffer, InsertZerosOps,
-                   InsertZerosOpsBuffered, DspVec, ToSliceMut, MetaData, Domain, RealNumberSpace,
-                   RededicateForceOps, ErrorReason};
+use numbers::*;
+use super::super::{ToComplexResult, TransRes, Buffer, InsertZerosOps, InsertZerosOpsBuffered,
+                   DspVec, ToSliceMut, MetaData, Domain, RealNumberSpace, RededicateForceOps,
+                   ErrorReason};
 
 /// Defines transformations from real to complex number space.
 ///
@@ -45,14 +45,15 @@ pub trait RealToComplexTransformsOpsBuffered<S, T>: ToComplexResult
     /// let result = vector.to_complex_b(&mut buffer);
     /// assert_eq!([1.0, 0.0, 2.0, 0.0], result[..]);
     /// ```
-    fn to_complex_b<B>(self, buffer: &mut B) -> Self::ComplexResult where B: Buffer<S, T>;
+    fn to_complex_b<B>(self, buffer: &mut B) -> Self::ComplexResult
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 impl<S, T, N, D> RealToComplexTransformsOps<T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToComplexResult + InsertZerosOps<T>,
           <DspVec<S, T, N, D> as ToComplexResult>::ComplexResult:
             RededicateForceOps<DspVec<S, T, N, D>>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: RealNumberSpace,
           D: Domain {
@@ -79,12 +80,12 @@ impl<S, T, N, D> RealToComplexTransformsOpsBuffered<S, T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToComplexResult + InsertZerosOpsBuffered<S, T>,
           <DspVec<S, T, N, D> as ToComplexResult>::ComplexResult:
             RededicateForceOps<DspVec<S, T, N, D>>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: RealNumberSpace,
           D: Domain {
       fn to_complex_b<B>(mut self, buffer: &mut B) -> Self::ComplexResult
-          where B: Buffer<S, T> {
+          where B: for<'a> Buffer<'a, S, T> {
           if self.is_complex() {
               self.number_space.to_complex();
               self.valid_len = 0;
