@@ -1,9 +1,8 @@
-use RealNumber;
 use numbers::*;
 use super::super::{ToTimeResult, ToRealTimeResult, TransRes, DspVec, Vector, Buffer, ToSliceMut,
-                   RededicateForceOps, ErrorReason, MetaData, ComplexNumberSpace, Owner,
-                   FrequencyDomain, DataDomain, InsertZerosOpsBuffered, ScaleOps,
-                   TimeDomainOperations, FrequencyDomainOperations};
+                   RededicateForceOps, ErrorReason, MetaData, ComplexNumberSpace, FrequencyDomain,
+                   DataDomain, InsertZerosOpsBuffered, ScaleOps, TimeDomainOperations,
+                   FrequencyDomainOperations};
 use super::fft;
 use window_functions::*;
 use multicore_support::*;
@@ -36,7 +35,7 @@ pub trait FrequencyToTimeDomainOperations<S, T>: ToTimeResult
     ///        assert!(f32::abs(actual[i] - expected[i]) < 1e-4);
     /// }
     /// ```
-    fn plain_ifft<B>(self, buffer: &mut B) -> Self::TimeResult where B: Buffer<S, T>;
+    fn plain_ifft<B>(self, buffer: &mut B) -> Self::TimeResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
     /// into a time domain vector.
@@ -55,12 +54,12 @@ pub trait FrequencyToTimeDomainOperations<S, T>: ToTimeResult
     ///        assert!(f32::abs(actual[i] - expected[i]) < 1e-4);
     /// }
     /// ```
-    fn ifft<B>(self, buffer: &mut B) -> Self::TimeResult where B: Buffer<S, T>;
+    fn ifft<B>(self, buffer: &mut B) -> Self::TimeResult where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
     /// into a time domain vector and removes the FFT window.
     fn windowed_ifft<B>(self, buffer: &mut B, window: &WindowFunction<T>) -> Self::TimeResult
-        where B: Buffer<S, T>;
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 /// Defines all operations which are valid on `DataVecs` containing frequency domain data and
@@ -83,7 +82,8 @@ pub trait SymmetricFrequencyToTimeDomainOperations<S, T>: ToRealTimeResult
     ///
     /// This version of the IFFT neither applies a window nor does it scale the
     /// vector.
-    fn plain_sifft<B>(self, buffer: &mut B) -> TransRes<Self::RealTimeResult> where B: Buffer<S, T>;
+    fn plain_sifft<B>(self, buffer: &mut B) -> TransRes<Self::RealTimeResult>
+        where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs a Symmetric Inverse Fast Fourier Transformation under the assumption that `self`
     /// contains half of a symmetric spectrum starting from 0 Hz. This assumption
@@ -92,7 +92,8 @@ pub trait SymmetricFrequencyToTimeDomainOperations<S, T>: ToRealTimeResult
     ///
     /// The argument indicates whether the resulting real vector should have `2*N` or
     /// `2*N-1` points.
-    fn sifft<B>(self, buffer: &mut B) -> TransRes<Self::RealTimeResult> where B: Buffer<S, T>;
+    fn sifft<B>(self, buffer: &mut B) -> TransRes<Self::RealTimeResult>
+        where B: for<'a> Buffer<'a, S, T>;
 
     /// Performs a Symmetric Inverse Fast Fourier Transformation (SIFFT) and removes the FFT
     /// window. The SIFFT is performed under the assumption that `self`
@@ -106,19 +107,19 @@ pub trait SymmetricFrequencyToTimeDomainOperations<S, T>: ToRealTimeResult
                          buffer: &mut B,
                          window: &WindowFunction<T>)
                          -> TransRes<Self::RealTimeResult>
-        where B: Buffer<S, T>;
+        where B: for<'a> Buffer<'a, S, T>;
 }
 
 impl<S, T, N, D> FrequencyToTimeDomainOperations<S, T> for DspVec<S, T, N, D>
     where DspVec<S, T, N, D>: ToTimeResult,
           <DspVec<S, T, N, D> as ToTimeResult>::TimeResult:
                 RededicateForceOps<DspVec<S, T, N, D>> + TimeDomainOperations<S, T>,
-          S: ToSliceMut<T> + Owner,
+          S: ToSliceMut<T>,
           T: RealNumber,
           N: ComplexNumberSpace,
           D: FrequencyDomain {
     fn plain_ifft<B>(mut self, buffer: &mut B) -> Self::TimeResult
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
           if self.domain() != DataDomain::Frequency {
               self.valid_len = 0;
               self.number_space.to_complex();
@@ -138,15 +139,15 @@ impl<S, T, N, D> FrequencyToTimeDomainOperations<S, T> for DspVec<S, T, N, D>
     }
 
     fn ifft<B>(mut self, buffer: &mut B) -> Self::TimeResult
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
           let points = self.points();
           self.scale(T::one() / T::from(points).unwrap());
-          self.ifft_shift(buffer);
+          self.ifft_shift();
           self.plain_ifft(buffer)
     }
 
     fn windowed_ifft<B>(self, buffer: &mut B, window: &WindowFunction<T>) -> Self::TimeResult
-      where B: Buffer<S, T> {
+      where B: for<'a> Buffer<'a, S, T> {
           let mut result = self.ifft(buffer);
           result.unapply_window(window);
           result
@@ -157,12 +158,12 @@ impl<S, T, N, D> SymmetricFrequencyToTimeDomainOperations<S, T> for DspVec<S, T,
      where DspVec<S, T, N, D>: ToRealTimeResult + ToTimeResult + FrequencyDomainOperations<S, T>,
            <DspVec<S, T, N, D> as ToRealTimeResult>::RealTimeResult:
                 RededicateForceOps<DspVec<S, T, N, D>> + TimeDomainOperations<S, T>,
-           S: ToSliceMut<T> + Owner,
+           S: ToSliceMut<T>,
            T: RealNumber,
            N: ComplexNumberSpace,
            D: FrequencyDomain {
    fn plain_sifft<B>(mut self, buffer: &mut B) -> TransRes<Self::RealTimeResult>
-       where B: Buffer<S, T> {
+       where B: for<'a> Buffer<'a, S, T> {
        if self.domain() != DataDomain::Frequency ||
           !self.is_complex() {
            self.valid_len = 0;
@@ -190,16 +191,16 @@ impl<S, T, N, D> SymmetricFrequencyToTimeDomainOperations<S, T> for DspVec<S, T,
    }
 
    fn sifft<B>(mut self, buffer: &mut B) -> TransRes<Self::RealTimeResult>
-       where B: Buffer<S, T> {
+       where B: for<'a> Buffer<'a, S, T> {
        let points = self.points();
        self.scale(Complex::<T>::new(T::one() / T::from(points).unwrap(), T::zero()));
-       self.ifft_shift(buffer);
+       self.ifft_shift();
        self.plain_sifft(buffer)
    }
 
    fn windowed_sifft<B>(self, buffer: &mut B, window: &WindowFunction<T>)
         -> TransRes<Self::RealTimeResult>
-       where B: Buffer<S, T> {
+       where B: for<'a> Buffer<'a, S, T> {
            let mut result = try!(self.sifft(buffer));
            result.unapply_window(window);
            Ok(result)
